@@ -40,7 +40,6 @@ struct TrackerMinerPrivate {
 	gboolean started;
 	
 	gchar *name;
-	gchar *description;
 	gchar *status;
 	gdouble progress;
 
@@ -62,7 +61,6 @@ typedef struct {
 enum {
 	PROP_0,
 	PROP_NAME,
-	PROP_DESCRIPTION,
 	PROP_STATUS,
 	PROP_PROGRESS
 };
@@ -178,13 +176,6 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 							      "Name",
 							      NULL,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (object_class,
-					 PROP_DESCRIPTION,
-					 g_param_spec_string ("description",
-							      "Description",
-							      "Description",
-							      NULL,
-							      G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
 					 PROP_STATUS,
 					 g_param_spec_string ("status",
@@ -241,10 +232,6 @@ miner_set_property (GObject      *object,
 		g_free (miner->private->name);
 		miner->private->name = g_value_dup_string (value);
 		break;
-	case PROP_DESCRIPTION:
-		g_free (miner->private->description);
-		miner->private->description = g_value_dup_string (value);
-		break;
 	case PROP_STATUS: {
 		const gchar *new_status;
 
@@ -293,9 +280,6 @@ miner_get_property (GObject    *object,
 	case PROP_NAME:
 		g_value_set_string (value, miner->private->name);
 		break;
-	case PROP_DESCRIPTION:
-		g_value_set_string (value, miner->private->description);
-		break;
 	case PROP_STATUS:
 		g_value_set_string (value, miner->private->status);
 		break;
@@ -314,7 +298,6 @@ miner_finalize (GObject *object)
 	TrackerMiner *miner = TRACKER_MINER (object);
 
 	g_free (miner->private->status);
-	g_free (miner->private->description);
 	g_free (miner->private->name);
 
 	if (miner->private->client) {
@@ -325,7 +308,7 @@ miner_finalize (GObject *object)
 		g_object_set_qdata (G_OBJECT (miner), dbus_data, NULL);
 	}
 
-	g_hash_table_destroy (miner->private->pauses);
+	g_hash_table_unref (miner->private->pauses);
 
 	G_OBJECT_CLASS (tracker_miner_parent_class)->finalize (object);
 }
@@ -365,6 +348,13 @@ miner_constructed (GObject *object)
 				 dbus_data_destroy);
 }
 
+/**
+ * tracker_miner_error_quark:
+ *
+ * Returns the #GQuark used to identify miner errors in GError structures.
+ *
+ * Returns: the error #GQuark
+ **/
 GQuark
 tracker_miner_error_quark (void)
 {
@@ -499,7 +489,7 @@ dbus_data_destroy (gpointer data)
 	}
 
 	if (dd->name_monitors) {
-		g_hash_table_destroy (dd->name_monitors);
+		g_hash_table_unref (dd->name_monitors);
 	}
 
 	g_slice_free (DBusData, dd);
@@ -592,6 +582,12 @@ pause_data_destroy (gpointer data)
 	g_slice_free (PauseData, pd);
 }
 
+/**
+ * tracker_miner_start:
+ * @miner: a #TrackerMiner
+ *
+ * Tells the miner to start processing data.
+ **/
 void
 tracker_miner_start (TrackerMiner *miner)
 {
@@ -603,6 +599,12 @@ tracker_miner_start (TrackerMiner *miner)
 	g_signal_emit (miner, signals[STARTED], 0);
 }
 
+/**
+ * tracker_miner_stop:
+ * @miner: a #TrackerMiner
+ *
+ * Tells the miner to stop processing data.
+ **/
 void
 tracker_miner_stop (TrackerMiner *miner)
 {
@@ -614,6 +616,14 @@ tracker_miner_stop (TrackerMiner *miner)
 	g_signal_emit (miner, signals[STOPPED], 0);
 }
 
+/**
+ * tracker_miner_is_started:
+ * @miner: a #TrackerMiner
+ *
+ * Returns #TRUE if the miner has been started.
+ *
+ * Returns: #TRUE if the miner is already started.
+ **/
 gboolean
 tracker_miner_is_started (TrackerMiner  *miner)
 {
@@ -622,6 +632,17 @@ tracker_miner_is_started (TrackerMiner  *miner)
 	return miner->private->started;
 }
 
+/**
+ * tracker_miner_execute_update:
+ * @miner: a #TrackerMiner
+ * @sparql: a SPARQL query
+ * @error: return location for errors
+ *
+ * Executes an update SPARQL query on tracker-store, use this
+ * whenever you want to perform data insertions or modifications.
+ *
+ * Returns: #TRUE if the SPARQL query was executed successfully.
+ **/
 gboolean
 tracker_miner_execute_update (TrackerMiner  *miner,
 			      const gchar   *sparql,
@@ -649,6 +670,18 @@ tracker_miner_execute_update (TrackerMiner  *miner,
 	return FALSE;
 }
 
+/**
+ * tracker_miner_execute_sparql:
+ * @miner: a #TrackerMiner
+ * @sparql: a SPARQL query
+ * @error: return location for errors
+ *
+ * Executes the SPARQL query on tracker-store and returns the
+ * queried data. Use this whenever you need to get data from
+ * already stored information.
+ *
+ * Returns: a #GPtrArray with the returned data.
+ **/
 GPtrArray *
 tracker_miner_execute_sparql (TrackerMiner  *miner,
 			      const gchar   *sparql,
@@ -677,6 +710,18 @@ tracker_miner_execute_sparql (TrackerMiner  *miner,
 	return res;
 }
 
+/**
+ * tracker_miner_execute_batch_update:
+ * @miner: a #TrackerMiner
+ * @sparql: a set of SPARQL updates
+ * @error: return location for errors
+ *
+ * Executes a batch of update SPARQL queries on tracker-store, use this
+ * whenever you want to perform data insertions or modifications in
+ * batches.
+ *
+ * Returns: #TRUE if the SPARQL query was executed successfully.
+ **/
 gboolean
 tracker_miner_execute_batch_update (TrackerMiner  *miner,
 				    const gchar   *sparql,
@@ -703,6 +748,14 @@ tracker_miner_execute_batch_update (TrackerMiner  *miner,
 	return FALSE;
 }
 
+/**
+ * tracker_miner_commit:
+ * @miner: a #TrackerMiner
+ *
+ * Commits all pending batch updates. see tracker_miner_execute_batch_update()
+ *
+ * Returns: #TRUE if the data was committed successfully.
+ **/
 gboolean
 tracker_miner_commit (TrackerMiner *miner)
 {
@@ -769,6 +822,18 @@ tracker_miner_pause (TrackerMiner  *miner,
 	return pd->cookie;
 }
 
+/**
+ * tracker_miner_resume:
+ * @miner: a #TrackerMiner
+ * @cookie: pause cookie
+ * @error: return location for errors
+ *
+ * Asks the miner to resume processing. The cookie must be something
+ * returned by tracker_miner_pause(). The miner won't actually resume
+ * operations until all pause requests have been resumed.
+ *
+ * Returns: #TRUE if the cookie was valid.
+ **/
 gboolean 
 tracker_miner_resume (TrackerMiner  *miner,
 		      gint           cookie,
@@ -792,42 +857,6 @@ tracker_miner_resume (TrackerMiner  *miner,
 }
 
 /* DBus methods */
-void
-tracker_miner_dbus_get_name (TrackerMiner           *miner,
-			     DBusGMethodInvocation  *context,
-			     GError                **error)
-{
-	guint request_id;
-
-	request_id = tracker_dbus_get_next_request_id ();
-
-	tracker_dbus_async_return_if_fail (miner != NULL, context);
-
-	tracker_dbus_request_new (request_id, "%s()", __PRETTY_FUNCTION__);
-
-	dbus_g_method_return (context, miner->private->name);
-
-	tracker_dbus_request_success (request_id);
-}
-
-void
-tracker_miner_dbus_get_description (TrackerMiner           *miner,
-				    DBusGMethodInvocation  *context,
-				    GError                **error)
-{
-	guint request_id;
-
-	request_id = tracker_dbus_get_next_request_id ();
-
-	tracker_dbus_async_return_if_fail (miner != NULL, context);
-
-	tracker_dbus_request_new (request_id, "%s()", __PRETTY_FUNCTION__);
-
-	dbus_g_method_return (context, miner->private->description);
-
-	tracker_dbus_request_success (request_id);
-}
-
 void
 tracker_miner_dbus_get_status (TrackerMiner           *miner,
 			       DBusGMethodInvocation  *context,
