@@ -76,8 +76,12 @@ typedef struct {
 	GSList   *ignored_directories;
 	GSList   *ignored_directories_with_content;
 	GSList   *ignored_files;
+
+	/* Convenience data */
 	GSList   *ignored_directory_patterns;
+	GSList   *ignored_directory_paths;
 	GSList   *ignored_file_patterns;
+	GSList   *ignored_file_paths;
 } TrackerConfigPrivate;
 
 typedef struct {
@@ -277,13 +281,24 @@ tracker_config_class_init (TrackerConfigClass *klass)
 					 PROP_INDEX_RECURSIVE_DIRECTORIES,
 					 g_param_spec_pointer ("index-recursive-directories",
 							       "Index recursive directories",
-							       " List of directories to crawl recursively for indexing (separator=;)",
+							       " List of directories to crawl recursively for indexing (separator=;)\n"
+							       " Special values include: (see /etc/xdg/user-dirs.defaults & $HOME/.config/user-dirs.default)\n"
+							       "   &DESKTOP\n"
+							       "   &DOCUMENTS\n"
+							       "   &DOWNLOAD\n"
+							       "   &MUSIC\n"
+							       "   &PICTURES\n"
+							       "   &PUBLIC_SHARE\n"
+							       "   &TEMPLATES\n"
+							       "   &VIDEOS\n"
+							       " If $HOME is the default below, it is because $HOME/.config/user-dirs.default was missing.",
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (object_class,
 					 PROP_INDEX_SINGLE_DIRECTORIES,
 					 g_param_spec_pointer ("index-single-directories",
 							       "Index single directories",
-							       " List of directories to index but not sub-directories for changes (separator=;)",
+							       " List of directories to index but not sub-directories for changes (separator=;)\n"
+							       " Special values used for IndexRecursiveDirectories can also be used here",
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (object_class,
 					 PROP_IGNORED_DIRECTORIES,
@@ -485,11 +500,21 @@ config_finalize (GObject *object)
 			 NULL);
 	g_slist_free (priv->ignored_file_patterns);
 
+	g_slist_foreach (priv->ignored_file_paths,
+			 (GFunc) g_free,
+			 NULL);
+	g_slist_free (priv->ignored_file_paths);
+
 	g_slist_foreach (priv->ignored_directory_patterns,
 			 (GFunc) g_pattern_spec_free,
 			 NULL);
 	g_slist_free (priv->ignored_directory_patterns);
 
+	g_slist_foreach (priv->ignored_directory_paths,
+			 (GFunc) g_free,
+			 NULL);
+	g_slist_free (priv->ignored_directory_paths);
+
 	g_slist_foreach (priv->ignored_files, (GFunc) g_free, NULL);
 	g_slist_free (priv->ignored_files);
 
@@ -563,15 +588,76 @@ config_create_with_defaults (TrackerConfig *config,
 		case G_TYPE_POINTER:
 			/* Special case string lists */
 			if (g_strcmp0 (conversions[i].property, "index-recursive-directories") == 0) {
-				const gchar *string_list[] = { NULL, NULL };
+				/* Since we can't know if
+				 * $HOME/.config/user-dirs.dirs exists
+				 * or not, we check that the GLib API
+				 * returns meaningful data.
+				 */
+				GUserDirectory dir;
+				GSList *paths = NULL;
+				GStrv string_list;
 
-				string_list[0] = g_get_home_dir ();
+				for (dir = 0; dir < G_USER_N_DIRECTORIES; dir++) {
+					const gchar *path;
+
+					path = g_get_user_special_dir (dir);
+					if (path == NULL || *path == '\0') {
+						continue;
+					}
+
+					switch (dir) {
+					case G_USER_DIRECTORY_DESKTOP:
+						path = "&DESKTOP";
+						break;
+					case G_USER_DIRECTORY_DOCUMENTS:
+						path = "&DOCUMENTS";
+						break;
+					case G_USER_DIRECTORY_DOWNLOAD:
+						path = "&DOWNLOAD";
+						break;
+					case G_USER_DIRECTORY_MUSIC:
+						path = "&MUSIC";
+						break;
+					case G_USER_DIRECTORY_PICTURES:
+						path = "&PICTURES";
+						break;
+					case G_USER_DIRECTORY_VIDEOS:
+						path = "&VIDEOS";
+						break;
+
+					/* We are not interested in
+					 * TEMPLATES or PUBLIC_SHAREs
+					 */
+					case G_USER_DIRECTORY_PUBLIC_SHARE:
+					case G_USER_DIRECTORY_TEMPLATES:
+					case G_USER_N_DIRECTORIES:
+						continue;
+					}
+
+					paths = g_slist_prepend (paths, (gpointer) path);
+				}
+
+				paths = g_slist_reverse (paths);
+
+				/* If we only found DESKTOP which is
+				 * always defined it seems, then add
+				 * $HOME.
+				 */
+				if (g_slist_length (paths) < 2) {
+					g_slist_free (paths);
+					paths = g_slist_prepend (NULL, "$HOME");
+				}
+
+				string_list = tracker_gslist_to_string_list (paths);
+				g_slist_free (paths);
 
 				g_key_file_set_string_list (key_file, 
 							    conversions[i].group, 
 							    conversions[i].key, 
-							    string_list, 
-							    G_N_ELEMENTS (string_list));
+							    (const gchar * const *) string_list, 
+							    g_strv_length (string_list));
+
+				g_strfreev (string_list);
 			} else if (g_strcmp0 (conversions[i].property, "ignored-directories") == 0) {
 				const gchar *string_list[] = { 
 					"po", "CVS", ".svn", ".git", "core-dumps", "lost+found",
@@ -637,11 +723,11 @@ config_create_with_defaults (TrackerConfig *config,
 }
 
 static void
-config_set_ignored_file_patterns (TrackerConfig *config)
+config_set_ignored_file_conveniences (TrackerConfig *config)
 {
 	TrackerConfigPrivate *priv;
-	GPatternSpec *spec;
 	GSList *l;
+	GSList *paths = NULL;
 	GSList *patterns = NULL;
 
 	priv = TRACKER_CONFIG_GET_PRIVATE (config);
@@ -651,23 +737,39 @@ config_set_ignored_file_patterns (TrackerConfig *config)
 			 NULL);
 	g_slist_free (priv->ignored_file_patterns);
 
+	g_slist_foreach (priv->ignored_file_paths,
+			 (GFunc) g_free,
+			 NULL);
+	g_slist_free (priv->ignored_file_paths);
+
 	for (l = priv->ignored_files; l; l = l->next) {
-		/* g_message ("  Adding file ignore pattern:'%s'", */
-		/* 	   (gchar *) l->data); */
-		spec = g_pattern_spec_new (l->data);
-		patterns = g_slist_prepend (patterns, spec);
+		const gchar *str = l->data;
+
+		if (!str) {
+			continue;
+		}
+
+		if (G_LIKELY (*str != G_DIR_SEPARATOR)) {
+			GPatternSpec *spec;
+
+			spec = g_pattern_spec_new (l->data);
+			patterns = g_slist_prepend (patterns, spec);
+		} else {
+			paths = g_slist_prepend (paths, g_strdup (l->data));
+		}
 	}
 
 	priv->ignored_file_patterns = g_slist_reverse (patterns);
+	priv->ignored_file_paths = g_slist_reverse (paths);
 }
 
 static void
-config_set_ignored_directory_patterns (TrackerConfig *config)
+config_set_ignored_directory_conveniences (TrackerConfig *config)
 {
 	TrackerConfigPrivate *priv;
-	GPatternSpec *spec;
 	GSList *l;
 	GSList *patterns = NULL;
+	GSList *paths = NULL;
 
 	priv = TRACKER_CONFIG_GET_PRIVATE (config);
 
@@ -676,14 +778,30 @@ config_set_ignored_directory_patterns (TrackerConfig *config)
 			 NULL);
 	g_slist_free (priv->ignored_directory_patterns);
 
+	g_slist_foreach (priv->ignored_directory_paths,
+			 (GFunc) g_free,
+			 NULL);
+	g_slist_free (priv->ignored_directory_paths);
+
 	for (l = priv->ignored_directories; l; l = l->next) {
-		/* g_message ("  Adding directory ignore pattern:'%s'", */
-		/* 	   (gchar *) l->data); */
-		spec = g_pattern_spec_new (l->data);
-		patterns = g_slist_prepend (patterns, spec);
+		const gchar *str = l->data;
+
+		if (!str) {
+			continue;
+		}
+
+		if (G_LIKELY (*str != G_DIR_SEPARATOR)) {
+			GPatternSpec *spec;
+
+			spec = g_pattern_spec_new (l->data);
+			patterns = g_slist_prepend (patterns, spec);
+		} else {
+			paths = g_slist_prepend (paths, g_strdup (l->data));
+		}
 	}
 
 	priv->ignored_directory_patterns = g_slist_reverse (patterns);
+	priv->ignored_directory_paths = g_slist_reverse (paths);
 }
 
 static void
@@ -725,12 +843,11 @@ config_load (TrackerConfig *config)
 							     conversions[i].key);
 			break;
 
-		case G_TYPE_POINTER:
-			if (strcmp (conversions[i].property, "ignored-files") == 0) {
-				is_directory_list = FALSE;
-			} else {
-				is_directory_list = TRUE;
-			}
+		case G_TYPE_POINTER: {
+			GSList *dirs, *l;
+			gboolean check_for_duplicates = FALSE;
+
+			is_directory_list = TRUE;
 
 			tracker_keyfile_object_load_string_list (G_OBJECT (file), 
 								 conversions[i].property,
@@ -738,12 +855,62 @@ config_load (TrackerConfig *config)
 								 conversions[i].group, 
 								 conversions[i].key,
 								 is_directory_list);
+
+			if (strcmp (conversions[i].property, "index-recursive-directories") != 0 &&
+			    strcmp (conversions[i].property, "index-single-directories") != 0) {
+				continue;
+			}
+			
+			g_object_get (config, conversions[i].property, &dirs, NULL);
+
+			for (l = dirs; l; l = l->next) {
+				const gchar *path_to_use;
+
+				/* Must be a special dir */
+				if (strcmp (l->data, "&DESKTOP") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP);
+				} else if (strcmp (l->data, "&DOCUMENTS") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS);
+				} else if (strcmp (l->data, "&DOWNLOAD") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DOWNLOAD);
+				} else if (strcmp (l->data, "&MUSIC") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_MUSIC);
+				} else if (strcmp (l->data, "&PICTURES") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
+				} else if (strcmp (l->data, "&PUBLIC_SHARE") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_PUBLIC_SHARE);
+				} else if (strcmp (l->data, "&TEMPLATES") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_TEMPLATES);
+				} else if (strcmp (l->data, "&VIDEOS") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_VIDEOS);
+				} else {
+					path_to_use = NULL;
+				}
+
+				if (path_to_use) {
+					check_for_duplicates = TRUE;
+					g_free (l->data);
+					l->data = g_strdup (path_to_use);
+				}
+			}
+
+			if (check_for_duplicates) {
+				GSList *filtered;
+
+				filtered = tracker_path_list_filter_duplicates (dirs, ".");
+				g_object_set (config, conversions[i].property, filtered, NULL);
+
+				g_slist_foreach (filtered, (GFunc) g_free, NULL);
+				g_slist_free (filtered);
+			}
+
 			break;
 		}
+		}
 	}
 
-	config_set_ignored_file_patterns (config);
-	config_set_ignored_directory_patterns (config);
+	config_set_ignored_file_conveniences (config);
+	config_set_ignored_directory_conveniences (config);
 }
 
 static gboolean
@@ -781,6 +948,52 @@ config_save (TrackerConfig *config)
 			break;
 
 		case G_TYPE_POINTER:
+			if (strcmp (conversions[i].property, "index-recursive-directories") == 0 ||
+			    strcmp (conversions[i].property, "index-single-directories") == 0) {
+				GSList *dirs, *l;
+
+				g_object_get (config, conversions[i].property, &dirs, NULL);
+				
+				for (l = dirs; l; l = l->next) {
+					const gchar *dir;
+					const gchar *path_to_use;
+
+					/* FIXME: This doesn't work
+					 * perfectly, what if DESKTOP
+					 * and DOCUMENTS are in the
+					 * same place? Then this
+					 * breaks. Need a better
+					 * solution at some point.
+					 */
+
+					dir = g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP);
+					if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP)) == 0) {
+						path_to_use = "&DESKTOP";
+					} else if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS)) == 0) {
+						path_to_use = "&DOCUMENTS";
+					} else if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_DOWNLOAD)) == 0) {
+						path_to_use = "&DOWNLOAD";
+					} else if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_MUSIC)) == 0) {
+						path_to_use = "&MUSIC";
+					} else if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_PICTURES)) == 0) {
+						path_to_use = "&PICTURES";
+					} else if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_PUBLIC_SHARE)) == 0) {
+						path_to_use = "&PUBLIC_SHARE";
+					} else if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_TEMPLATES)) == 0) {
+						path_to_use = "&TEMPLATES";
+					} else if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_VIDEOS)) == 0) {
+						path_to_use = "&VIDEOS";
+					} else {
+						path_to_use = NULL;
+					}
+					
+					if (path_to_use) {
+						g_free (l->data);
+						l->data = g_strdup (path_to_use);
+					}
+				}
+			}
+
 			tracker_keyfile_object_save_string_list (file,
 								 conversions[i].property, 
 								 file->key_file,
@@ -1290,6 +1503,9 @@ tracker_config_set_ignored_directories (TrackerConfig *config,
 	g_slist_foreach (l, (GFunc) g_free, NULL);
 	g_slist_free (l);
 
+	/* Re-set up the GPatternSpec list */
+	config_set_ignored_directory_conveniences (config);
+
 	g_object_notify (G_OBJECT (config), "ignored-directories");
 }
 
@@ -1342,6 +1558,9 @@ tracker_config_set_ignored_files (TrackerConfig *config,
 	g_slist_foreach (l, (GFunc) g_free, NULL);
 	g_slist_free (l);
 
+	/* Re-set up the GPatternSpec list */
+	config_set_ignored_file_conveniences (config);
+
 	g_object_notify (G_OBJECT (config), "ignored-files");
 }
 
@@ -1372,3 +1591,27 @@ tracker_config_get_ignored_file_patterns (TrackerConfig *config)
 
 	return priv->ignored_file_patterns;
 }
+
+GSList *
+tracker_config_get_ignored_directory_paths (TrackerConfig *config)
+{
+	TrackerConfigPrivate *priv;
+
+	g_return_val_if_fail (TRACKER_IS_CONFIG (config), NULL);
+
+	priv = TRACKER_CONFIG_GET_PRIVATE (config);
+
+	return priv->ignored_directory_paths;
+}
+
+GSList *
+tracker_config_get_ignored_file_paths (TrackerConfig *config)
+{
+	TrackerConfigPrivate *priv;
+
+	g_return_val_if_fail (TRACKER_IS_CONFIG (config), NULL);
+
+	priv = TRACKER_CONFIG_GET_PRIVATE (config);
+
+	return priv->ignored_file_paths;
+}
