@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2008-2009, Nokia
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,22 +28,15 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <gio/gio.h>
 
-#include <libtracker/tracker.h>
+#include <libtracker-client/tracker.h>
 #include <libtracker-common/tracker-common.h>
 
-static gchar	     *service;
-static gchar        **uri = NULL;
+static gchar        **filenames = NULL;
 
 static GOptionEntry   entries[] = {
-	{ "service", 's', 0, G_OPTION_ARG_STRING, &service,
-	  N_("Service type of the file"),
-	  NULL
-	},
-	{ G_OPTION_REMAINING, 0,
-	  G_OPTION_FLAG_FILENAME, G_OPTION_ARG_STRING_ARRAY, &uri,
-	  N_("FILE..."),
+	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &filenames,
+	  N_("FILE"),
 	  N_("FILE")},
 	{ NULL }
 };
@@ -58,16 +51,30 @@ print_property_value (gpointer value)
 	g_print ("  '%s' = '%s'\n", pair[0], pair[1]);
 }
 
+static gboolean
+has_valid_uri_scheme (const gchar *uri)
+{
+	const gchar *s;
+
+	s = uri;
+
+	if (!g_ascii_isalpha (*s)) {
+		return FALSE;
+	}
+
+	do {
+		s++;
+	} while (g_ascii_isalnum (*s) || *s == '+' || *s == '.' || *s == '-');
+
+	return (*s == ':');
+}
+
 int
 main (int argc, char **argv)
 {
 	TrackerClient	*client;
-	ServiceType	 type;
-	GFile           *file;
-	gchar           *abs_path;
 	GOptionContext	*context;
-	GError		*error = NULL;
-	GPtrArray	*results;
+	gchar          **p;
 
 	setlocale (LC_ALL, "");
 
@@ -77,18 +84,18 @@ main (int argc, char **argv)
 
 	/* Translators: this messagge will apper immediately after the	*/
 	/* usage string - Usage: COMMAND [OPTION]... <THIS_MESSAGE>	*/
-	context = g_option_context_new (_("- Get all information from a certain file"));
+	context = g_option_context_new (_("- Get all information about one or more files"));
 
 	/* Translators: this message will appear after the usage string */
 	/* and before the list of options.				*/
 	g_option_context_add_main_entries (context, entries, NULL);
 	g_option_context_parse (context, &argc, &argv, NULL);
 
-	if (!uri) {
+	if (!filenames) {
 		gchar *help;
 
 		g_printerr ("%s\n\n",
-			    _("Uri missing"));
+			    _("One or more files have not been specified"));
 
 		help = g_option_context_get_help (context, TRUE, NULL);
 		g_option_context_free (context);
@@ -100,67 +107,72 @@ main (int argc, char **argv)
 
 	g_option_context_free (context);
 
-	client = tracker_connect (FALSE);
+	client = tracker_connect (FALSE, G_MAXINT);
 
 	if (!client) {
 		g_printerr ("%s\n",
-			    _("Could not establish a DBus connection to Tracker"));
+			    _("Could not establish a D-Bus connection to Tracker"));
 		return EXIT_FAILURE;
 	}
 
-	if (!service) {
-		g_print ("%s\n",
-			 _("Defaulting to 'files' service"));
+        for (p = filenames; *p; p++) {
+		GPtrArray *results;
+		GError    *error = NULL;
+		gchar     *uri;
+		gchar     *query;
 
-		type = SERVICE_FILES;
-	} else {
-		type = tracker_service_name_to_type (service);
-
-		if (type == SERVICE_OTHER_FILES && g_ascii_strcasecmp (service, "Other")) {
-			g_printerr ("%s\n",
-				    _("Service type not recognized, using 'Other' ..."));
-		}
-	}
-
-	file = g_file_new_for_commandline_arg (uri[0]);
-	abs_path = g_file_get_path (file);
-
-	results = tracker_metadata_get_all (client,
-					    type,
-					    abs_path,
-					    &error);
-	g_free (abs_path);
-	g_object_unref (file);
+		g_print ("%s:'%s'\n",
+			 _("Querying information for file"),
+			 *p);
 	
-	if (error) {
-		g_printerr ("%s, %s\n",
-			    _("Unable to retrieve data for uri"),
-			    error->message);
+		/* support both, URIs and local file paths */
+		if (has_valid_uri_scheme (*p)) {
+			uri = g_strdup (*p);
+		} else {
+			GFile *file;
+			
+			file = g_file_new_for_commandline_arg (*p);
+			uri = g_file_get_uri (file);
+			g_object_unref (file);
+		}
 		
-		g_error_free (error);
-		tracker_disconnect (client);
+		query = g_strdup_printf ("SELECT ?predicate ?object WHERE { <%s> ?predicate ?object }", uri);
 		
-		return EXIT_FAILURE;
-	}
+		results = tracker_resources_sparql_query (client, query, &error);
+		
+		g_free (uri);
+		g_free (query);
 
-	if (!results) {
-		g_print ("%s\n",
-			 _("No metadata available for that uri"));
-	} else {
-		gint length;
+		if (error) {
+			g_printerr ("  %s, %s\n",
+				    _("Unable to retrieve data for URI"),
+				    error->message);
+			
+			g_error_free (error);
+			continue;
+		}
+		
+		if (!results) {
+			g_print ("  %s\n",
+				 _("No metadata available for that URI"));
+		} else {
+			gint length;
+			
+			length = results->len;
+			
+			g_print (tracker_dngettext (NULL,
+						    "Result: %d", 
+						    "Results: %d",
+						    length),
+				 length);
+			g_print ("\n");
+			
+			g_ptr_array_foreach (results, (GFunc) print_property_value, NULL);
+			g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
+			g_ptr_array_free (results, TRUE);
+		}
 
-		length = results->len;
-
-		g_print (tracker_dngettext (NULL,
-					    _("Result: %d"), 
-					    _("Results: %d"),
-					    length),
-			 length);
 		g_print ("\n");
-		
-		g_ptr_array_foreach (results, (GFunc) print_property_value, NULL);
-		g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
-		g_ptr_array_free (results, TRUE);
 	}
 
 	tracker_disconnect (client);
